"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globalMetadataStorage = void 0;
exports.createMetadataStorage = createMetadataStorage;
exports.getGlobalMetadataStorage = getGlobalMetadataStorage;
exports.getType = getType;
exports.defineType = defineType;
const rootTypes_1 = require("./definition/rootTypes");
const scalar_1 = require("./definition/scalar");
const MetadataStorage_1 = require("./metadata/MetadataStorage");
const coerceType_1 = require("./type-expression/coerceType");
/**
 * Create a new MetadataStorage initialized with default metadata
 */
function createMetadataStorage() {
    const storage = new MetadataStorage_1.MetadataStorage();
    (0, scalar_1.loadBuiltInScalar)(storage);
    (0, rootTypes_1.loadFallbackRootTypes)(storage);
    return storage;
}
function getGlobalMetadataStorage() {
    if (!exports.globalMetadataStorage) {
        exports.globalMetadataStorage = createMetadataStorage();
    }
    return exports.globalMetadataStorage;
}
/**
 * Get a GraphQLType instance from the given storage or default
 * global metadata storage.
 * @param typeArg
 * @param storage
 */
function getType(typeArg, kind = 'any') {
    const storage = getGlobalMetadataStorage();
    return (0, coerceType_1.coerceType)(typeArg).getType({ kind, storage });
}
/**
 * Define a type linked to decorated class and add it to the given
 * storage or default global metadata storage.
 * @param metadataOrThunk
 */
function defineType(parsersOrThunk) {
    const storage = getGlobalMetadataStorage();
    return function defDecoratorFn(definitionClass) {
        storage.deferRegister(() => {
            (Array.isArray(parsersOrThunk) ? parsersOrThunk : parsersOrThunk()).forEach(parser => {
                storage.registerMetadata(parser.parseWithResolverMap(definitionClass));
            });
        });
        return definitionClass;
    };
}
//# sourceMappingURL=global.js.map